﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;

namespace Tetris.Models;

/* It's a class that represents a game of Tetris */
public class Game : INotifyPropertyChanged {
    private Random _random = new Random();
    
    private string _userName;

    public string UserName {
        get => _userName;
        set {
            _userName = value;
            OnPropertyChanged("UserName");
        }
    }

    private int _score;
    
    public int Score {
        get => _score;
        set {
            _score = value;
            OnPropertyChanged("Score");
        }
    }

    private bool _playing = true;
    
    public bool Playing {
        get => _playing;
        set {
            _playing = value;
            OnPropertyChanged("Playing");
        }
    }

    private Grid _grid;

    public Grid Grid => _grid;

    private Tetrominoe? _currentTetrominoe;
    public Tetrominoe? CurrentTetrominoe => _currentTetrominoe;

    private Tetrominoe? _nextTetrominoe;
    public Tetrominoe? NextTetrominoe => _nextTetrominoe;

    /* It's the constructor of the class Game. */
    public Game(string userName, Grid grid) {
        _userName = userName;
        _grid = grid;
        _currentTetrominoe = GetNewTetrominoe();
        _nextTetrominoe = GetNewTetrominoe();
    }

    /// <summary>
    /// Return true if the current tetrominoe can't go down one
    /// row
    /// </summary>
    /// <returns>
    /// A boolean value.
    /// </returns>
    public bool HitBottom() {
        if (_currentTetrominoe == null)
            return false;

        return !_grid.CanGo(_currentTetrominoe, _currentTetrominoe.Coordinates + new Size(0, 1));
    }

    /// <summary>
    /// If the current tetrominoe can go down one, return false.
    /// Otherwise, return true if the current tetrominoe is at the top of the grid
    /// </summary>
    /// <returns>
    /// a boolean value.
    /// </returns>
    public bool HitTop() {
        if (_currentTetrominoe == null)
            return false;
        else if (_grid.CanGo(_currentTetrominoe, _currentTetrominoe.Coordinates + new Size(0, 1)))
            return false;

        return _grid.MinGrid.Y == _currentTetrominoe.Coordinates.Y;
    }

    /// <summary>
    /// It checks if there is a full line, if there is, it clears it and adds to the score
    /// </summary>
    public void ClearLine()
    {
        int line = _grid.LineFull();
        
        while (line != -1)
        {
            _grid.ClearLine(line);
            Score += Grid.CGrid.GetLength(0);
            line = _grid.LineFull();
        }
    }

    /// <summary>
    /// Print the tetrominoe to the grid, and if the game is playing, set the current tetrominoe
    /// to the next tetrominoe and get a new next tetrominoe
    /// </summary>
    /// <returns>
    /// The current tetrominoe is being returned.
    /// </returns>
    public void PrintTetrominoe() {
        if (_currentTetrominoe == null)
            return;
        
        _grid.PrintTetrominoe(_currentTetrominoe);
        
        if (Playing) {
            _currentTetrominoe = _nextTetrominoe;
            _nextTetrominoe = GetNewTetrominoe();
        } else {
            _currentTetrominoe = null;
            _nextTetrominoe = null;
        }
    }

    /// <summary>
    /// Get a random tetrominoe from the list of tetrominoe strings, and return a new Tetrominoe object with that string.
    /// </summary>
    /// <returns>
    /// A new Tetrominoe object.
    /// </returns>
    public Tetrominoe GetNewTetrominoe() {
        List<String> tetrominoes = TetrominoeParser.List();
        return new Tetrominoe(_grid, tetrominoes[_random.Next(tetrominoes.Count)]);
    }

    /// <summary>
    /// Save the score to the scoreboard
    /// </summary>
    public void SaveGame() {
        Console.WriteLine("Game over");
        using var db = new ScoreContext();
        var score = new Score{Points = Score, UserName = UserName};
        db.Scores.Add(score);
        db.SaveChanges();
    }

    public event PropertyChangedEventHandler PropertyChanged;

    protected virtual void OnPropertyChanged(string propertyName)
    {
        if (PropertyChanged != null)
            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
    }
}
