﻿using System;
using System.Drawing;
using System.Linq;
using System.Windows.Documents;
using System.Windows.Media;
using Pastel;
using Color = System.Drawing.Color;

namespace Tetris.Models; 

/* It's a 2D array of colors */
public class Grid {
    private Color[,] _grid;

    public Color[,] CGrid => _grid;

    public Point MinGrid => new Point(0,0);
    public Point MaxGrid => new Point(_grid.GetLength(0)-1, _grid.GetLength(1)-1);

    /// <summary>
    /// > Create a Grid object with a given grid of colors
    /// </summary>
    /// <param name="grid">The color grid.</param>
    public Grid(Color[,] grid) {
        _grid = grid;
    }

    /// <summary>
    /// > If the tetrominoe is within the grid and doesn't overlap with any other tetrominoe, then return true
    /// </summary>
    /// <param name="Tetrominoe">The tetrominoe to check if it can go to the point.</param>
    /// <param name="Point">The point where the tetrominoe will be placed.</param>
    /// <returns>
    /// A boolean value.
    /// </returns>
    public bool CanGo(Tetrominoe tetrominoe, Point point) {
        bool[,] shape = tetrominoe.Shape;
        if (point.X < MinGrid.X || point.Y < MinGrid.Y)
            return false;
        
        else if ((point.X + tetrominoe.Shape.GetLength(0)-1) > MaxGrid.X || (point.Y  + tetrominoe.Shape.GetLength(1)-1) > MaxGrid.Y)
            return false;

        for (uint x = 0; x < shape.GetLength(0); x++)
            for (uint y = 0; y < shape.GetLength(1); y++) {
                Point s = point + new Size((int) x, (int) y);
                if (shape[x, y] && _grid[s.X, s.Y] != Color.Empty)
                    return false;
            }
        
        return true;
    }
    
    /// <summary>
    /// > If the line is full, return the line number, otherwise return -1
    /// </summary>
    /// <returns>
    /// The y value of the first full line.
    /// </returns>
    public int LineFull()
    {
        for (int y = MaxGrid.Y; y > 0; y--)
        {
            bool full = true;
            for (int x = 0; x <= MaxGrid.X; x++)
                if (_grid[x, y] == Color.Empty)
                {
                    full = false;
                    break;
                }
            if (full)
                return y;
        }

        return -1;
    }

    /// <summary>
    /// > For each column, move all the blocks down one row, and then clear the top row
    /// </summary>
    /// <param name="line">The line to clear</param>
    public void ClearLine(int line) {
        for (int x = MinGrid.X; x <= MaxGrid.X; x++)
            for (int y = line; y > MinGrid.Y; y--) {
                _grid[x, y] = _grid[x, y - 1];
            }
        
        for (int x = 0; x <= MaxGrid.X; x++)
            _grid[x,MinGrid.Y] = Color.Empty;
    }

    /// <summary>
    /// For each point in the shape, add the shape's coordinates to the point, and if the point is true, set the grid's
    /// color to the shape's color.
    /// </summary>
    /// <param name="Tetrominoe">The Tetrominoe to be printed.</param>
    public void PrintTetrominoe(Tetrominoe t) {
        bool[,] shape = t.Shape;
        
        for (uint x = 0; x < shape.GetLength(0); x++)
            for (uint y = 0; y < shape.GetLength(1); y++) {
                Point s = t.Coordinates + new Size((int) x, (int) y);
                if (shape[x, y])
                    _grid[s.X, s.Y] = t.Color;
            }
    }

    /// <summary>
    /// Covert the gid to a color string
    /// </summary>
    /// <returns>
    /// A string representation of the grid.
    /// </returns>
    public override string ToString() {
        String s = "";
        for (uint y = 0; y < _grid.GetLength(1); y++) {
            for (uint x = 0; x < _grid.GetLength(0); x++) {
                s += "x".Pastel(_grid[x, y]);
            }
            s += "\n";
        }
        return s;
    }

    /// <summary>
    /// Covert the gid to a color string with the current tetrominoe
    /// </summary>
    /// <param name="Tetrominoe">The tetrominoe to print.</param>
    /// <returns>
    /// A string representation of the grid.
    /// </returns>
    public string ToString(Tetrominoe t) {
        Grid g = new((Color[,]) _grid.Clone());
        g.PrintTetrominoe(t);
        return g.ToString();
    }
}
