﻿using System;
using System.ComponentModel;
using System.Drawing;
using Pastel;

namespace Tetris.Models; 

/* It's a representation of a Tetrominoe, which is a Tetris piece */
public class Tetrominoe {
    public event PropertyChangedEventHandler PropertyChanged;

    private Grid _grid;
    
    private bool[,] _shape;
    public bool[,] Shape {
        get {
            bool[,] shape = (bool[,]) _shape.Clone();
            for (int i = 0; i < _orientation; i++)
                shape = _rotateLeft(shape);
            return shape;
        }
    }
    
    private Point _coordinates;
    public Point Coordinates {
        get => _coordinates;
        set {
            if (! _grid.CanGo(this, value))
                return;
            _coordinates = value;
            OnPropertyChanged("Coordinates");
        }
    }
    
    private short _orientation = 0;
    public short Orientation {
        get => _orientation;
        set {
            if (value > 3)
                value = 0;
            else if (value < 0)
                value = 3;

            short oldOrientation = _orientation;
            _orientation = value;

            Point newCoords = _coordinates;
            //ToDo: move position when hit a wall
            
            if (!_grid.CanGo(this, newCoords)) {
                _orientation = oldOrientation;
            } else {
                OnPropertyChanged("Orientation");
            }
        }
    }

    private Color _color;
    public Color Color {
        get => _color;
    }
    
    /* Create a tetrominoe of a grid from a shape with coordinates, an orientation and a color */
    public Tetrominoe(Grid grid, bool[,] shape, Point coordinates, short orientation, Color color) {
        _grid = grid;
        _shape = shape;
        _coordinates = coordinates;
        _orientation = orientation;
        _color = color;
    }
    
    /* Generate an tetrominoe automatically from his name */
    public Tetrominoe(Grid grid, String name) {
        _grid = grid;
        _shape = TetrominoeParser.GetShape(name);
        _coordinates = new Point(Convert.ToInt32(Math.Floor(_grid.CGrid.GetLength(0) / 2.0) - Math.Floor(_shape.GetLength(0) / 2.0)), 0);
        // ToDo fixme, not rounded to left
        _orientation = 0;
        _color = TetrominoeParser.GetColor(name);
    }

    /// <summary>
    /// > We're taking the original array and rotating it 90 degrees to the left
    /// </summary>
    /// <param name="shape">The shape to be rotated.</param>
    /// <returns>
    /// A new array with the same dimensions as the original array, but rotated 90 degrees to the left.
    /// </returns>
    private bool[,] _rotateLeft(bool[,] shape) {
        bool[,] rotatedArr = new bool[shape.GetLength(1), shape.GetLength(0)];
        
        for (int i=shape.GetLength(0)-1;i>=0;--i)
            for (int j=0;j<shape.GetLength(1);++j)
                rotatedArr[j,(shape.GetLength(0)-1)-i] = shape[i,j];

        return rotatedArr;
    }
    
    public void RotateLeft() {
        Orientation -= 1;
    }

    public void RotateRight() {
        Orientation += 1;
    }

    public void GoRight() {
        Coordinates += new Size(1, 0);
    }

    public void GoLeft() {
        Coordinates -= new Size(1, 0);
    }

    public void GoDown() {
        Coordinates += new Size(0, 1);
    }
    
    protected virtual void OnPropertyChanged(string propertyName) {
        if (PropertyChanged != null)
            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
    }

    public override string ToString() {
        String s = "";
        for (uint y = 0; y < Shape.GetLength(1); y++) {
            for (uint x = 0; x < Shape.GetLength(0); x++)
                if (Shape[x, y])
                    s += "x".Pastel(_color);
                else
                    s += "-".Pastel(Color.Black);
            s += "\n";
        }
        return s;
    }
}
